﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Drawing;

namespace howto_fill_circle_overlap
{
    public class ArcInfo
    {
        public PointF Point1, Point2, Center;
        public RectangleF Rect;
        public float StartAngle, SweepAngle;

        public ArcInfo(PointF center, PointF point1, PointF point2)
        {
            Center = center;
            Point1 = point1;
            Point2 = point2;

            float dx1 = Point1.X - Center.X;
            float dy1 = Point1.Y - Center.Y;
            float dx2 = Point2.X - Center.X;
            float dy2 = Point2.Y - Center.Y;

            // Get the radius and rectangle.
            float radius = (float)Math.Sqrt(dx1 * dx1 + dy1 * dy1);
            Rect = new RectangleF(
                Center.X - radius,
                Center.Y - radius,
                2 * radius, 2 * radius);

            // Calculate the angles.
            double angle1 = Math.Atan2(dy1, dx1);
            double angle2 = Math.Atan2(dy2, dx2);

            // Make angle2 > angle1.
            if (angle1 > angle2) angle2 += 2 * Math.PI;

            // If the sweep is more thn 180 degrees, swap them.
            double sweep = angle2 - angle1;
            if (sweep > Math.PI)
            {
                Swap(ref angle1, ref angle2);
                Swap(ref Point1, ref Point2);
                angle2 += 2 * Math.PI;
                sweep = angle2 - angle1;
            }

            StartAngle = (float)(angle1 * 180 / Math.PI);
            SweepAngle = (float)((angle2 - angle1) * 180 / Math.PI);
        }

        public void Swap<T>(ref T a, ref T b)
        {
            T temp = a;
            a = b;
            b = temp;
        }

        public void Draw(Graphics gr, Pen pen)
        {
            gr.DrawArc(pen, Rect, StartAngle, SweepAngle);
        }
    }
}
